@@ -1,6 +1,6 @@
 package MyPageKit::Common;
 
-# $Id: Common.pm,v 1.6 2001/05/04 14:50:35 tjmather Exp $
+# $Id: Common.pm,v 1.7 2001/05/13 03:42:36 tjmather Exp $
 
 use strict;
 
@@ -9,10 +9,11 @@ use vars qw(@ISA);
 
 use Apache::Constants qw(OK REDIRECT DECLINED);
 
-$Apache::PageKit::secret_md5 = 'you_should_place_your_own_md5_string_here';
+$MyPageKit::Common::secret_md5 = 'you_should_place_your_own_md5_string_here';
 
 use DBI;
 use MyPageKit::MyModel;
+use Digest::MD5;
 
 sub pkit_dbi_connect {
   # this line should be replaced with a DBI->connect(...) statement
@@ -30,6 +31,8 @@ sub pkit_session_setup {
 		       session_store_class => 'File',
 		       session_lock_class => 'File',
 		       session_args => {
+#					Handle => $dbh,
+#					LockHandle => $dbh,
 					Directory => '/tmp/pkit_sessions',
 					LockDirectory => '/tmp/pkit_sessions_lock',
 				       }
@@ -45,22 +48,17 @@ sub pkit_common_code {
 
   # for the pagekit.org website, we control the colors based on the
   # values the user selected, stored in the session.
-  $model->output_param(link_color => $session->{'link_color'} || 'ff9933');
-  $model->output_param(text_color => $session->{'text_color'} || '000000');
-  $model->output_param(bgcolor => $session->{'bgcolor'} || 'dddddd');
-  $model->output_param(mod_color => $session->{'mod_color'} || 'ffffff');
-
-  # for testing/illustration purposes only:
-  # uncomment the following code to enable editing on the website 
-  # for any user
-  #$model->output_param(pkit_admin => 1);
+  $model->output(link_color => $session->{'link_color'} || 'ff9933');
+  $model->output(text_color => $session->{'text_color'} || '000000');
+  $model->output(bgcolor => $session->{'bgcolor'} || 'dddddd');
+  $model->output(mod_color => $session->{'mod_color'} || 'ffffff');
 }
 
 sub pkit_auth_credential {
   my ($model) = @_;
   my $dbh = $model->dbh;
-  my $login = $model->input_param('login');
-  my $passwd = $model->input_param('passwd');
+  my $login = $model->input('login');
+  my $passwd = $model->input('passwd');
 
   unless ($login ne "" && $passwd ne ""){
     $model->pkit_message("You did not fill all of the fields.  Please try again.",
@@ -79,7 +77,7 @@ sub pkit_auth_credential {
     return;
   }
 
-  my $hash = Digest::MD5->md5_hex(join ':', $Apache::MyPageKit::secret_md5, $user_id, $epasswd);
+  my $hash = Digest::MD5->md5_hex(join ':', $MyPageKit::Common::secret_md5, $user_id, $epasswd);
 
   my $ses_key = {
 		 'user_id'   => $user_id,
@@ -102,11 +100,13 @@ sub pkit_auth_session_key {
 
   # create a new hash and verify that it matches the supplied hash
   # (prevents tampering with the cookie)
-  my $newhash = Digest::MD5->md5_hex(join ':', $Apache::MyPageKit::secret_md5, $user_id, crypt($epasswd,"pk"));
+  my $newhash = Digest::MD5->md5_hex(join ':', $MyPageKit::Common::secret_md5, $user_id, crypt($epasswd,"pk"));
 
   return unless $newhash eq $ses_key->{'hash'};
 
-  $model->output_param('pkit_login',$login);
+  $model->output(pkit_user => $user_id);
+
+  $model->output('pkit_login',$login);
 
   return $user_id;
 }
@@ -186,7 +186,6 @@ a MD5 hash of a a secret key, user ID, password.
 
 Should verify the session key (previously generated by C<auth_credential>)
 and return the user ID.
-This user ID will be fed to C<$model-E<gt>input_param('pkit_user')>.
 
 =item pkit_common_code
 
