@@ -1,6 +1,6 @@
 package Apache::PageKit::View;
 
-# $Id: View.pm,v 1.48 2001/05/07 17:34:59 tjmather Exp $
+# $Id: View.pm,v 1.52 2001/05/13 03:42:36 tjmather Exp $
 
 # we want to extend this module to use different templating packages -
 # Template::ToolKit and HTML::Template
@@ -26,6 +26,7 @@ use Data::Dumper;
 
 # 2. template objects are loaded and stored on disk or in memory, in
 # a hash containing fields from the following set:
+#    * exclude_params - array ref of lists of params to be excluded from <PKIT_SELFURL> tags
 #    * html_template - HTML::Template object
 #    * template_toolkit - Template-Tookit object
 #    * filename - filename of template source
@@ -33,15 +34,17 @@ use Data::Dumper;
 #        (contains all of the files included by the <PKIT_COMPONENT> tags
 #    * component_ids - an array ref containing component_ids that have
 #        code associated with them
+#    * has_form - 1 if contains <form> tag, 0 otherwise.  used to
+#        determine whether to apply HTML::FillInForm module
 # the objects themselves are keyed by page_id, pkit_view and lang
 
 # 3. methods that are called externally
-#    * new($view_dir, $content_dir, $cache_dir, $default_lang, $reload, $html_clean_level, $can_edit, [ $associated_objects ], [ $fill_in_form_objects] ) (args passed as hash reference)
+#    * new($view_dir, $content_dir, $cache_dir, $default_lang, $reload, $html_clean_level, $can_edit, [ $associated_objects ], [ $fillinform_objects], $input_param_object, $output_param_object) (args passed as hash reference)
 #    * fill_in_view($page_id, $pkit_view, $lang)
 #    * open_view($page_id, $pkit_view, $lang)
 #    * param
 #    * preparse_templates($view_root,$html_clean_level,$view_cache_dir);
-#    * template_file_exists($page_id)
+#    * template_file_exists($page_id, $pkit_view)
 
 # 4. methods that are called interally
 #    * _fetch_from_file_cache($page_id, $pkit_view, $lang);
@@ -68,6 +71,21 @@ sub fill_in_view {
 
   my $tmpl = $record->{html_template};
 
+  # Fill in (compiled) <PKIT_SELFURL> tags
+  my $exclude_params_set = $record->{exclude_params_set};
+  if($exclude_params_set && @$exclude_params_set){
+    my $input_param_object = $view->{input_param_object};
+    my $orig_uri = $input_param_object->notes('orig_uri');
+    foreach my $exclude_params (@$exclude_params_set){
+      my @exclude_params = split(" ",$exclude_params);
+      my $query_string = Apache::PageKit::params_as_string($input_param_object, \@exclude_params);
+      if($query_string){
+	$tmpl->param("pkit_selfurl$exclude_params", ($orig_uri . '?' . $query_string) . '&');
+      } else {
+	$tmpl->param("pkit_selfurl$exclude_params", $orig_uri . '?');
+      }
+    }
+  }
   # fill in data from associated objects (for example from the Apache request
   # object if $apr is set
   foreach my $object (@{$view->{associated_objects}}){
@@ -80,32 +98,31 @@ sub fill_in_view {
     }
   }
 
-  # finally, we use the $view object to fill in template
+  # finally, we use the $output_param_object object to fill in template
   # get params from $view object
   # note that in this case we allow for MODEL_LOOPs as well as MODEL_VARs
-  foreach my $key ($view->param){
-    my $value = $view->param($key);
+  my $output_param_object = $view->{output_param_object};
+  foreach my $key ($output_param_object->param){
+    my $value = $output_param_object->param($key);
     $tmpl->param($key, $value);
   }
 
   my $output = $tmpl->output;
 
-  # if fill_in_form_objects is set, then we use that to fill in any HTML
-  # forms in the template.
-  my $fif;
-  if(@{$view->{fill_in_form_objects}}){
-    $fif = HTML::FillInForm->new();
-  }
-
-  foreach my $object (@{$view->{fill_in_form_objects}}){
-    $output = $fif->fill(scalarref=>\$output,
-			 fobject => $object);
+  if($record->{has_form}){
+    # if fillinform_objects is set, then we use that to fill in any HTML
+    # forms in the template.
+    my $fif;
+    if(@{$view->{fillinform_objects}}){
+      $fif = HTML::FillInForm->new();
+      $output = $fif->fill(scalarref=>\$output,
+			   fobject => $view->{fillinform_objects} );
+    }
   }
 
   if(exists $INC{'Apache/PKCMS/View.pm'}){
     Apache::PKCMS::View::add_edit_links($view, $record, \$output);
   }
-
   return \$output;
 }
 
@@ -137,38 +154,6 @@ sub open_view {
   $view->{already_loaded}->{$page_id} = 1;
 }
 
-# param method - can be called in two forms
-# when passed two arguments ($name, $value), it sets the value of the 
-# $name attributes to $value
-# when passwd one argument ($name), retrives the value of the $name attribute
-# used to access and set values of <MODEL_*> tags
-sub param {
-  my ($view, @p) = @_;
-
-  unless(@p){
-    # the no-parameter case - return list of parameters
-    return () unless defined($view) && $view->{'pkit_parameters'};
-    return () unless @{$view->{'pkit_parameters'}};
-    return @{$view->{'pkit_parameters'}};
-  }
-  my ($name, $value);
-  # deal with case of setting mul. params with hash ref.
-  if (ref($p[0]) eq 'HASH'){
-    my $hash_ref = shift(@p);
-    push(@p, %$hash_ref);
-  }
-  if (@p > 1){
-    die "param called with odd number of parameters" unless ((@p % 2) == 0);
-    while(($name, $value) = splice(@p, 0, 2)){
-      $view->_add_parameter($name);
-      $view->{pkit_param}->{$name} = $value;
-    }
-  } else {
-    $name = $p[0];
-  }
-  return $view->{pkit_param}->{$name};
-}
-
 sub preparse_templates {
   my ($view) = @_;
 
@@ -183,31 +168,24 @@ sub preparse_templates {
     $view->open_view($page_id, $pkit_view);
   };
 
-  File::Find::find({wanted => $load_template_sub,
-		    follow => 1},
+  File::Find::find({wanted => $load_template_sub},
+#		    follow => 1},
 		    $view_dir);
 }
 
 sub template_file_exists {
   my ($view, $page_id, $pkit_view) = @_;
-  return 1 if $view->_find_template($pkit_view,'Page',$page_id);
+  return 1 if $view->_find_template($pkit_view,$page_id);
 }
 
 # private methods
 
-sub _add_parameter {
-  my ($view, $param) = @_;
-  return unless defined $param;
-  push (@{$view->{'pkit_parameters'}},$param)
-    unless defined($view->{$param});
-}
-
 sub _fetch_from_file_cache {
   my ($view, $page_id, $pkit_view, $lang) = @_;
 
-  my ($extra_param, $param_hash);
+  my ($extra_param, $param_hash) = ("","");
   if (my @xml_params = sort keys %{$Apache::PageKit::Content::PAGE_ID_XSL_PARAMS->{$page_id}}){
-    my $param_obj = $view->{input_param_obj};
+    my $param_obj = $view->{input_param_object};
     for my $xml_param (@xml_params){
       $extra_param .= "&$xml_param=" . $param_obj->param($xml_param);
     }
@@ -231,12 +209,12 @@ sub _fetch_from_file_cache {
 }
 
 sub _find_template {
-  my ($view, $pkit_view, $type, $id) = @_;
-  my $template_file = "$view->{view_dir}/$pkit_view/$type/$id.tmpl";
+  my ($view, $pkit_view, $id) = @_;
+  my $template_file = "$view->{view_dir}/$pkit_view/$id.tmpl";
   if(-e "$template_file"){
     return $template_file;
   } else {
-    $template_file = "$view->{view_dir}/Default/$type/$id.tmpl";
+    $template_file = "$view->{view_dir}/Default/$id.tmpl";
     if(-e "$template_file"){
       return $template_file;
     } else {
@@ -253,8 +231,8 @@ sub _html_clean {
 
   return unless $html_clean_level > 0;
 
-  my $h = new HTML::Clean($html_code_ref,$Apache::PageKit::View::html_clean_level) || die("can't open HTML::Clean object: $!");
-  $h->strip if $Apache::PageKit::View::html_clean_level > 0;
+  my $h = new HTML::Clean($html_code_ref,$html_clean_level) || die("can't open HTML::Clean object: $!");
+  $h->strip if $html_clean_level > 0;
   $$html_code_ref = ${$h->data()};
 }
 
@@ -262,13 +240,13 @@ sub _html_clean {
 sub _include_components {
   my ($view, $page_id, $html_code_ref, $pkit_view) = @_;
 
-  $$html_code_ref =~ s/<PKIT_COMPONENT (NAME=)?"?(.*?)"?>/&get_component($2,$view,$pkit_view)/eig;
+  $$html_code_ref =~ s!<PKIT_COMPONENT (NAME=)?"?(.*?)"?>(</PKIT_COMPONENT>)?!&get_component($page_id,$2,$view,$pkit_view)!eig;
 
 #  my @component_ids = keys %component_ids;
 #  return \@component_ids;
 
   sub get_component {
-    my ($component_id, $view, $pkit_view) = @_;
+    my ($page_id,$component_id, $view, $pkit_view) = @_;
 
     # check for recursive pkit_components
     $view->{component_ids_hash}->{$component_id}++;
@@ -276,7 +254,7 @@ sub _include_components {
       die "Likely recursive PKIT_COMPONENTS for component_id $component_id and giving up.";
     }
 
-    my $template_ref = $view->_load_component('Component',$page_id,$component_id,$pkit_view);
+    my $template_ref = $view->_load_component($page_id,$component_id,$pkit_view);
     return $$template_ref;
   }
 }
@@ -301,7 +279,7 @@ sub _is_record_uptodate {
 
     if($filename =~ m!^$view->{view_dir}/Default/! && $pkit_view ne 'Default'){
       # check to see if any new files have been uploaded to the $pkit_view dir
-      (my $check_filename = $filename)=~ s!^$view->{view_dir}/Default/!$view->{view_dir}/$pkit_view!;
+      (my $check_filename = $filename) =~ s!^$view->{view_dir}/Default/!$view->{view_dir}/$pkit_view/!;
       if (-e "$check_filename"){
 	return 0;
       }
@@ -314,15 +292,15 @@ sub _is_record_uptodate {
 
 # here the usage of "component" also includes page
 sub _load_component {
-  my ($view, $type, $page_id, $component_id, $pkit_view) = @_;
+  my ($view, $page_id, $component_id, $pkit_view) = @_;
 
-  my $template_file = $view->_find_template($pkit_view, $type, $component_id);
+  my $template_file = $view->_find_template($pkit_view, $component_id);
   my $template_ref;
 
   unless($template_file){
     # no template file exists, attempt to generate from XML and XSL files
     # currently only XML::LibXSLT is supported
-    $template_ref = $view->{content}->generate_template($type, $page_id, $component_id, $pkit_view, $view->{input_param_obj});
+    $template_ref = $view->{content}->generate_template($page_id, $component_id, $pkit_view, $view->{input_param_object});
   } else {
     open TEMPLATE, "$template_file";
     local($/) = undef;
@@ -357,17 +335,19 @@ sub _load_page {
   $view->{include_mtimes} = $content->{include_mtimes} = {};
   $view->{component_ids_hash} = {};
 
-  my $template_file = $view->_find_template($pkit_view, 'Page', $page_id);
-  my $template_ref = $view->_load_component('Page',$page_id,$page_id,$pkit_view);
+  my $template_file = $view->_find_template($pkit_view, $page_id);
+  my $template_ref = $view->_load_component($page_id,$page_id,$pkit_view);
 
-#  my $template_file = $view->_find_template($pkit_view, 'Page', $page_id);
+#  my $template_file = $view->_find_template($pkit_view, $page_id);
   my $lang_tmpl = $content->process_template($page_id, $template_ref);
 
   # go through content files (which have had content filled in)
   while (my ($lang, $filtered_html) = each %$lang_tmpl){
-    $view->_preparse_model_tags($filtered_html);
+    my $exclude_params_set = $view->_preparse_model_tags($filtered_html);
     $view->_html_clean($filtered_html);
 
+    my $has_form = ($$filtered_html =~ m!<form!i);
+
     my $tmpl;
     eval {
       $tmpl = HTML::Template->new(scalarref => $filtered_html,
@@ -383,10 +363,12 @@ sub _load_page {
     }
     my @component_ids = keys %{$view->{component_ids_hash}};
     my $record = {
+		  exclude_params_set => $exclude_params_set,
 		  filename => $template_file,
 		  html_template => $tmpl,
 		  include_mtimes => $view->{include_mtimes},
-		  component_ids => \@component_ids
+		  component_ids => \@component_ids,
+		  has_form => $has_form
 		 };
 
     # make directories, if approriate
@@ -396,9 +378,9 @@ sub _load_page {
       File::Path::mkpath("$view->{cache_dir}/$dir");
     }
 
-    my ($extra_param, $param_hash);
+    my ($extra_param, $param_hash) = ("", "");
     if (my @xml_params = sort keys %{$Apache::PageKit::Content::PAGE_ID_XSL_PARAMS->{$page_id}}){
-      my $param_obj = $view->{input_param_obj};
+      my $param_obj = $view->{input_param_object};
       for my $xml_param (@xml_params){
 	$extra_param .= "&$xml_param=" . $param_obj->param($xml_param);
       }
@@ -419,15 +401,33 @@ sub _load_page {
 sub _preparse_model_tags {
   my ($view, $html_code_ref) = @_;
 
+  my $exclude_params_set = {};
+
   # "compile" PageKit templates into HTML::Templates
   $$html_code_ref =~ s!<MODEL_(VAR|LOOP|IF|ELSE|UNLESS)!<TMPL_$1!sig;
   $$html_code_ref =~ s!</MODEL_(LOOP|IF|UNLESS)!</TMPL_$1!sig;
 
+  # tags generated by XSLT
+  $$html_code_ref =~ s!</(MODEL|PKIT)_VAR>!!ig;
+
+  $$html_code_ref =~ s!<PKIT_SELFURL( +exclude=('|")(.*?)('|"))? *>!&process_selfurl_tag($exclude_params_set, $3)!eig;
+
   $$html_code_ref =~ s!<PKIT_ERRORFONT (NAME=)?"?([^"]*?)"?>(.*?)</PKIT_ERRORFONT>!<TMPL_VAR NAME="PKIT_ERRORFONT_BEGIN_$2">$3<TMPL_VAR NAME="PKIT_ERRORFONT_END_$2">!sig;
   $$html_code_ref =~ s!<PKIT_(VAR|LOOP|IF|UNLESS) +NAME *= *("?)__(FIRST|INNER|ODD|LAST)!<TMPL_$1 NAME=$2__$3!sig;
   $$html_code_ref =~ s!<PKIT_(VAR|LOOP|IF|UNLESS) +NAME *= *("?)!<TMPL_$1 NAME=$2PKIT_!sig;
   $$html_code_ref =~ s!<PKIT_ELSE!<TMPL_ELSE!sig;
   $$html_code_ref =~ s!</PKIT_(LOOP|IF|UNLESS)!</TMPL_$1!sig;
+
+  my @a = keys %$exclude_params_set;
+  return \@a;
+
+  sub process_selfurl_tag {
+    my ($exclude_params_set, $exclude_params) = @_;
+    $exclude_params = defined($exclude_params) ? 
+      join(" ",sort split("\s+",$exclude_params)) : "";
+    %$exclude_params_set->{$exclude_params} = 1;
+    return qq{<TMPL_VAR NAME="pkit_selfurl$exclude_params">};
+  }
 }
 
 1;
